# Makefile -- pdflatex makefile rules
#
# Copyright (C) 2008 Frank Langbein <frank@langbein.org>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 675 Mass Ave, Cambridge, MA 02139, USA.
#

PDFLATEX  = pdflatex
BIBTEX	  = bibtex
MAKEINDEX = makeindex
EGREP     = egrep
TEXDEP    = texdep
HANDOUT   = cftalk-handout

RERUN     = "(There were undefined references|Rerun to get (cross-references|the bars) right)"
RERUNBIB  = "No file.*\.bbl|Citation.*undefined"
MAKEIDX   = "^[^%]*\\makeindex"

COPY = if test -r $(<:%.tex=%.toc); then cp $(<:%.tex=%.toc) $(<:%.tex=%.toc.bak); fi 
RM = rm -f
OUTDATED = echo "EPS-file is out-of-date!" && false

%.pdf : %.tex
	TEXINPUTS="`(cd .. && pwd)`:$$TEXINPUTS" $(PDFLATEX) $<
	$(EGREP) $(MAKEIDX) $< && ($(MAKEINDEX) $(<:%.tex=%) && $(PDFLATEX) $<); true
	$(EGREP) -c $(RERUNBIB) $(<:%.tex=%.log) && ($(BIBTEX) $(<:%.tex=%) && $(PDFLATEX) $<); true
	$(EGREP) $(RERUN) $(<:%.tex=%.log) && $(PDFLATEX) $<; true
	$(EGREP) $(RERUN) $(<:%.tex=%.log) && $(PDFLATEX) $<; true
	$(PDFLATEX) $<

%-handout.pdf : %.pdf
	$(HANDOUT) $<

%.d : %.tex
	echo -n "$*.pdf $*-handout.pdf: " >$*.d
	TEXINPUTS="`(cd .. && pwd)`:$$TEXINPUTS" $(TEXDEP) $< >>$*.d

%.otl: %.pdf

clean::
	$(RM) *.log *.aux *.dvi *.bbl *.blg *.ilg *.toc *.lof *.lot *.idx *.ind *.otl *.out *.d *~
.PHONY: clean
