// cube.cc

// Compile: c++ -o cube cube.cc -L/usr/X11R6/lib -lm -lX11 -lXext -lXt -lXmu -lXi -lGL -lGLU -lglut

#include <stdlib.h>
#include <unistd.h>
#include <GL/glut.h>
#include <math.h>

// Degrees to radians conversion factor
const double c = 3.14159 / 180.0;

// Rotation angles
static GLfloat theta[] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};

// Current rotation axis
static GLint axis = 0;

// Spin?
static bool sp = false;

// Cube material
static GLfloat diffuse[] = {0.7, 0.0, 0.0, 1.0};
static GLfloat ambient[] = {0.6, 0.0, 0.0, 1.0};
static GLfloat specular[] = {1.0, 1.0, 1.0, 1.0};
static GLfloat shine = 127.0;

// Delta angle for rendering
static double da = 2.0;

// Light dot material
static GLfloat diffusel[] = {1.0, 1.0, 1.0, 1.0};
static GLfloat ambientl[] = {1.0, 1.0, 1.0, 1.0};
static GLfloat specularl[] = {0.0, 0.0, 0.0, 1.0};
static GLfloat shinel = 1.0;

// Light source 0
static GLfloat diffuse0[] = {1.0, 1.0, 1.0, 1.0};
static GLfloat ambient0[] = {0.2, 0.2, 0.2, 1.0};
static GLfloat specular0[] = {1.0, 1.0, 1.0, 1.0};
static GLfloat light0_pos[] = {3, 3, 3, 1.0};
static GLfloat attenuation0 = 0.8;

// Draw a coloured cube
void cube ()
{
  // Material
  glMaterialfv (GL_FRONT, GL_AMBIENT, ambient);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, diffuse);
  glMaterialfv (GL_FRONT, GL_SPECULAR, specular);
  glMaterialfv (GL_FRONT, GL_SHININESS, &shine);

  glBegin (GL_POLYGON);
  glNormal3f ( 0.0,  0.0, -1.0);
  glVertex3f (-1.0,  1.0, -1.0);
  glVertex3f (-1.0, -1.0, -1.0);
  glVertex3f ( 1.0, -1.0, -1.0);
  glVertex3f ( 1.0,  1.0, -1.0);
  glEnd ();

  glBegin (GL_POLYGON);
  glNormal3f ( 1.0,  0.0,  0.0);
  glVertex3f ( 1.0,  1.0, -1.0);
  glVertex3f ( 1.0, -1.0, -1.0);
  glVertex3f ( 1.0, -1.0,  1.0);
  glVertex3f ( 1.0,  1.0,  1.0);
  glEnd ();

  glBegin (GL_POLYGON);
  glNormal3f ( 0.0,  0.0,  1.0);
  glVertex3f ( 1.0,  1.0,  1.0);
  glVertex3f ( 1.0, -1.0,  1.0);
  glVertex3f (-1.0, -1.0,  1.0);
  glVertex3f (-1.0,  1.0,  1.0);
  glEnd ();

  glBegin (GL_POLYGON);
  glNormal3f (-1.0,  0.0,  0.0);
  glVertex3f (-1.0,  1.0,  1.0);
  glVertex3f (-1.0, -1.0,  1.0);
  glVertex3f (-1.0, -1.0, -1.0);
  glVertex3f (-1.0,  1.0, -1.0);
  glEnd ();

  glBegin (GL_POLYGON);
  glNormal3f ( 0.0,  1.0,  0.0);
  glVertex3f ( 1.0,  1.0, -1.0);
  glVertex3f (-1.0,  1.0, -1.0);
  glVertex3f (-1.0,  1.0,  1.0);
  glVertex3f ( 1.0,  1.0,  1.0);
  glEnd ();

  glBegin (GL_POLYGON);
  glNormal3f ( 0.0, -1.0,  0.0);
  glVertex3f (-1.0, -1.0, -1.0);
  glVertex3f (-1.0, -1.0,  1.0);
  glVertex3f ( 1.0, -1.0,  1.0);
  glVertex3f ( 1.0, -1.0, -1.0);
  glEnd ();
  
  return;
}

// display callback, clear frame buffer and z buffer,
// rotate cube and draw, swap buffers
void display ()
{
  // Clear
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // Model view matrix mode
  glMatrixMode (GL_MODELVIEW);
  // Set to matrix to identity
  glLoadIdentity ();

  // Transformations here woudl rotate the camera with the
  // light source

  // Camera Position (first matrix, i.e. last transformation)
  gluLookAt (0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

  // Everything specified here stays fixed with respect to 
  // camera position

  // Point indicating position of light source (fixed)
  glMaterialfv (GL_FRONT, GL_AMBIENT, ambientl);
  glMaterialfv (GL_FRONT, GL_DIFFUSE, diffusel);
  glMaterialfv (GL_FRONT, GL_SPECULAR, specularl);
  glMaterialfv (GL_FRONT, GL_SHININESS, &shinel);
  glPointSize (10.0);
  glBegin (GL_POINTS);
  glVertex4fv (light0_pos);
  glEnd ();

  // Light rotates with object (fixed light source position)
  glLightfv (GL_LIGHT0, GL_POSITION, light0_pos);
 
  // Cube model transformation (to rotate the cube)
  glRotatef (theta[0], 1.0, 0.0, 0.0);
  glRotatef (theta[1], 0.0, 1.0, 0.0);
  glRotatef (theta[2], 0.0, 0.0, 1.0);

  // Draw coloured cube
  cube ();

  // Flush
  glFlush ();

  // Swap buffers (double buffering for smooth animation)
  glutSwapBuffers ();
}

// Idle callback, spin cube about selected axis
void spin ()
{
  if (sp) {
    theta[axis] += 3.0;
    if (theta[axis] > 360.0)
      theta[axis] -= 360.0;
    glutPostRedisplay ();
  }

  return;
}


// mouse callback, selects an axis about which to rotate
void mouse (int btn, int state, int x, int y)
{
  if (btn == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
    theta[axis] += 10.0;

  if (btn == GLUT_MIDDLE_BUTTON && state == GLUT_DOWN)
    axis = (axis == 2) ? 0 : (axis + 1);

  if (btn == GLUT_RIGHT_BUTTON && state == GLUT_DOWN)
    sp = !sp;

  glutPostRedisplay ();
  return;
}

// Window reshape callback
void myReshape (int w, int h)
{
  glViewport (0, 0, w, h);

  // Projection matrix mode
  glMatrixMode (GL_PROJECTION);

  // Set to identity
  glLoadIdentity ();
 
  // Set perspective frustum
  glFrustum (-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);

  // Set different projection mode here

  return;
}

int main(int argc, char **argv)
{
  glutInit (&argc, argv);

  // need both double buffering and z buffer
  glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

  glutInitWindowSize (500, 500);
  glutCreateWindow ("cube");

  glutReshapeFunc (myReshape);
  glutDisplayFunc (display);
  glutIdleFunc (spin);
  glutMouseFunc (mouse);

  // Clear color
  glClearColor (0.0, 0.0, 0.0, 1.0);

  // Enable hidden-surface-removal
  glDepthFunc (GL_LEQUAL);
  glClearDepth (1.0);
  glEnable (GL_DEPTH_TEST); 
  
  // Enable lighting and one light source
  glEnable (GL_LIGHTING);
  glEnable (GL_LIGHT0);
  glLightfv (GL_LIGHT0, GL_AMBIENT, ambient0);
  glLightfv (GL_LIGHT0, GL_DIFFUSE, diffuse0);
  glLightfv (GL_LIGHT0, GL_SPECULAR, specular0);
  glLightfv (GL_LIGHT0, GL_CONSTANT_ATTENUATION, &attenuation0);

  // No two-sided polygons
  glLightModeli (GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);

  // Local viewer for specular light?
  glLightModeli (GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);

  // Enable smoooth shading (multi-coloured polygons)
  glShadeModel (GL_SMOOTH);

  // Fill polygons
  glPolygonMode(GL_FRONT, GL_FILL);

  // Initiate main loop
  glutMainLoop();

  return 0;
}
