// sphere.cc

// Compile: c++ -o sphere sphere.cc -L/usr/X11R6/lib -lm -lX11 -lXext -lXt -lXmu -lXi -lGL -lGLU -lglut

#include <stdlib.h>
#include <unistd.h>
#include <GL/glut.h>
#include <math.h>

// Degrees to radians conversion factor
const double c = 3.14159 / 180.0;

// Rotation angles
static GLfloat theta[] = {0.0,0.0,0.0};

// Current rotation axis
static GLint axis = 0;

// Spin?
static bool sp = false;

// Draw a sphere
void sphere ()
{
  // Quadrilateral strips
  for (double phi = -80.0; phi <= 80.0; phi += 10.0) {
    glBegin (GL_QUAD_STRIP);

    for (double thet = -180.0; thet <= 180.0; thet += 10.0) {

      glVertex3d (sin (c * thet) * cos (c * phi),
		  cos (c * thet) * cos (c * phi),
		  sin (c * phi)); 

      glVertex3d (sin (c * thet) * cos (c * (phi + 10.0)),
		  cos (c * thet) * cos (c * (phi + 10.0)),
		  sin (c * (phi + 10.0)));  
    }

    glEnd();
  }

  // North pole
  glBegin (GL_TRIANGLE_FAN);

  glVertex3d (0.0, 0.0, 1.0);

  for (double thet = -180.0; thet <= 180.0; thet += 10.0)
    glVertex3d (sin (c * thet) * cos (c * 80.0),
		cos (c * thet) * cos (c * 80.0),
		sin (c * 80.0));

  glEnd();

  // South pole
  glBegin (GL_TRIANGLE_FAN);
  
  glVertex3d (0.0, 0.0, -1.0);

  for (double thet = -180.0; thet <= 180.0; thet += 10.0)
    glVertex3d (sin (c * thet) * cos (c * 80.0),
		cos (c * thet) * cos (c * 80.0),
		sin (c * -80.0));

  glEnd();

  return;
}

// display callback, clear frame buffer and z buffer,
// rotate cube and draw, swap buffers
void display ()
{
  // Clear
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // Rotation matrix
  glLoadIdentity ();
  glRotatef (theta[0], 1.0, 0.0, 0.0);
  glRotatef (theta[1], 0.0, 1.0, 0.0);
  glRotatef (theta[2], 0.0, 0.0, 1.0);

  // Draw sphere
  sphere ();

  // Flush
  glFlush ();

  // Swap buffers (double buffering for smooth animation)
  glutSwapBuffers ();
}

// Idle callback, spin sphere 1 degrees about selected axis
void spin ()
{
  if (sp) {
    theta[axis] += 1.0;
    if (theta[axis] > 360.0)
      theta[axis] -= 360.0;
    glutPostRedisplay ();
  }

  return;
}

// mouse callback, selects an axis about which to rotate */
void mouse (int btn, int state, int x, int y)
{
  if (btn == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
    theta[axis] += 10;

  if (btn == GLUT_MIDDLE_BUTTON && state == GLUT_DOWN)
    axis = (axis == 2) ? 0 : (axis + 1);

  if (btn == GLUT_RIGHT_BUTTON && state == GLUT_DOWN)
    sp = !sp;

  glutPostRedisplay ();
  return;
}

// Window reshape callback
void myReshape (int w, int h)
{
  glViewport (0, 0, w, h);
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
 
  // Orthographiuc projection
  GLfloat a = static_cast<GLfloat> (h) / static_cast<GLfloat> (w);
  if (w <= h)
    glOrtho (-2.0, 2.0, -2.0 * a, 2.0 * a, -10.0, 10.0);
  else
    glOrtho (-2.0 / a, 2.0 / a, -2.0, 2.0, -10.0, 10.0);

  glMatrixMode (GL_MODELVIEW);
}

int main(int argc, char **argv)
{
  glutInit (&argc, argv);

  // need both double buffering and z buffer
  glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

  glutInitWindowSize (500, 500);
  glutCreateWindow ("sphere");

  glutReshapeFunc (myReshape);
  glutDisplayFunc (display);
  glutIdleFunc (spin);
  glutMouseFunc (mouse);

  glEnable (GL_DEPTH_TEST); // Enable hidden--surface--removal
  glClearColor (1.0, 1.0, 1.0, 1.0);
  glColor3f (0.0, 0.0, 0.0);

  glPolygonMode(GL_FRONT_AND_BACK,GL_LINE); // Wireframe mode

  glutMainLoop();

  return 0;
}
